"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Service = void 0;
const error_1 = require("../helpers/tier0/error");
const generators_1 = require("./generators");
const resolverHelper = require("../resolvers/resolver");
const jql_1 = require("../helpers/tier0/jql");
const subscription_1 = require("../helpers/tier2/subscription");
const __1 = require("..");
class Service {
    static getTypeDef() {
        return __1.getTypeDefs()[this.__typename];
    }
    static async testPermissions(operation, req, args, query) {
        if (!req.cache) {
            req.cache = {};
        }
        //check if this operation was already done. if so, return that result
        if ((operation + '-' + this.__typename) in req.cache) {
            return req.cache[operation + '-' + this.__typename];
        }
        let allowed;
        if (this.accessControl && (operation in this.accessControl)) {
            allowed = await this.accessControl[operation](req, args, query);
        }
        else {
            allowed = true;
        }
        //cache the permissions check in req if not already there
        return (req.cache[operation + '-' + this.__typename] = allowed);
    }
    static async subscribeToSingleItem(operationName, req, args, query, admin = false) {
        const selectQuery = query || Object.assign({}, this.presets.default);
        //if it does not pass the access control, throw an error
        if (!admin && !await this.testPermissions('get', req, args, query)) {
            throw error_1.default.badPermissionsError();
        }
        //check if the record and query is fetchable
        const results = await resolverHelper.resolveTableRows(this.__typename, this, req, {
            select: selectQuery,
            where: [
                {
                    fields: [
                        { field: "id", value: args.id }
                    ]
                }
            ]
        }, args);
        if (results.length < 1) {
            throw error_1.default.itemNotFoundError();
        }
        const validatedArgs = {
            id: args.id
        };
        const channel = await subscription_1.handleJqlSubscription(req, operationName, validatedArgs, query || Object.assign({}, this.presets.default));
        return {
            channel_name: channel
        };
    }
    static async subscribeToMultipleItem(operationName, req, args, query, admin = false) {
        const selectQuery = query || Object.assign({}, this.presets.default);
        //if it does not pass the access control, throw an error
        if (!admin && !await this.testPermissions('getMultiple', req, args, query)) {
            throw error_1.default.badPermissionsError();
        }
        //check if the query is valid (no need to actually run it)
        jql_1.default.validateJsonqlQuery(selectQuery, this.__typename);
        const validatedArgs = {
            created_by: args.created_by
        };
        const channel = await subscription_1.handleJqlSubscription(req, operationName, validatedArgs, query || Object.assign({}, this.presets.default));
        return {
            channel_name: channel
        };
    }
    static async getRecord(req, args = {}, query, count = false, admin = false) {
        var _a;
        const selectQuery = query || Object.assign({}, this.presets.default);
        //if it does not pass the access control, throw an error
        if (!admin && !await this.testPermissions('get', req, args, query)) {
            throw error_1.default.badPermissionsError();
        }
        const filterArray = [];
        if (this.hasKeys) {
            //handle filter fields
            for (const arg in args) {
                if (arg in this.filterFieldsKeyMap) {
                    const filterObject = {
                        connective: "AND",
                        fields: []
                    };
                    filterObject.fields.push({
                        field: (_a = this.filterFieldsKeyMap[arg].field) !== null && _a !== void 0 ? _a : arg,
                        joinFields: this.filterFieldsKeyMap[arg].joinFields,
                        value: args[arg]
                    });
                    filterArray.push(filterObject);
                }
            }
            if (filterArray.length < 1) {
                throw error_1.default.generateError("Must supply at least 1 filter parameter");
            }
        }
        const results = await resolverHelper.resolveTableRows(this.__typename, this, req, {
            select: selectQuery,
            where: filterArray,
            limit: 1,
        }, args);
        if (results.length < 1) {
            throw error_1.default.itemNotFoundError();
        }
        return results[0];
    }
    static async getRecords(req, args = {}, query, count = false, admin = false) {
        var _a, _b;
        const selectQuery = query || Object.assign({}, this.presets.default);
        //if it does not pass the access control, throw an error
        if (!admin && !await this.testPermissions('getMultiple', req, args, query)) {
            throw error_1.default.badPermissionsError();
        }
        const filterArray = [];
        //handle filter fields
        for (const arg in args) {
            if (arg in this.filterFieldsMap) {
                const filterObject = {
                    connective: "AND",
                    fields: []
                };
                filterObject.fields.push({
                    field: (_a = this.filterFieldsMap[arg].field) !== null && _a !== void 0 ? _a : arg,
                    joinFields: this.filterFieldsMap[arg].joinFields,
                    value: args[arg]
                });
                filterArray.push(filterObject);
            }
        }
        //handle search fields
        if (args.search) {
            const filterObject = {
                connective: "OR",
                fields: []
            };
            for (const prop in this.searchFieldsMap) {
                filterObject.fields.push({
                    field: (_b = this.searchFieldsMap[prop].field) !== null && _b !== void 0 ? _b : prop,
                    joinFields: this.searchFieldsMap[prop].joinFields,
                    value: '%' + args.search + '%',
                    operator: 'LIKE'
                });
            }
            filterArray.push(filterObject);
        }
        if (filterArray.length < 1 && this.isFilterRequired) {
            throw error_1.default.generateError("Must supply at least 1 filter parameter");
        }
        if (count) {
            //add the "after" constraint, if provided
            if (args.after) {
                filterArray.push({
                    fields: {
                        id: { value: args.after, operator: '>' }
                    }
                });
            }
            const resultsCount = await resolverHelper.countTableRows(this.__typename, filterArray);
            return resultsCount;
        }
        else {
            //parse args.first and ensure it is less than 100
            const limit = parseInt(args.first) > 100 ? 100 : (parseInt(args.first) || 100);
            const results = await resolverHelper.resolveTableRows(this.__typename, this, req, {
                select: selectQuery,
                where: filterArray,
                orderBy: Array.isArray(args.sortBy) ? args.sortBy.reduce((total, item, index) => {
                    var _a;
                    if (item in this.sortFieldsMap) {
                        total.push({
                            field: (_a = this.sortFieldsMap[item].field) !== null && _a !== void 0 ? _a : item,
                            desc: Array.isArray(args.sortDesc) ? (args.sortDesc[index] === "true" || args.sortDesc[index] === true) : true
                        });
                    }
                    return total;
                }, []) : null,
                limit: limit,
                after: args.after,
                groupBy: Array.isArray(args.groupBy) ? args.groupBy.reduce((total, item, index) => {
                    var _a;
                    if (item in this.groupByFieldsMap) {
                        total.push({
                            field: (_a = this.groupByFieldsMap[item].field) !== null && _a !== void 0 ? _a : item
                        });
                    }
                    return total;
                }, []) : null,
            }, args);
            return results;
        }
    }
    static async createRecord(req, args = {}, query, admin = false) {
        if (!req.user)
            throw error_1.default.loginRequiredError();
        //if it does not pass the access control, throw an error
        if (!admin && !await this.testPermissions('create', req, args, query)) {
            throw error_1.default.badPermissionsError();
        }
        const addResults = await resolverHelper.addTableRow(this.__typename, Object.assign({}, args), {
            created_by: req.user.id
        });
        const validatedArgs = {
            created_by: req.user.id
        };
        subscription_1.handleJqlSubscriptionTriggerIterative(req, this, this.__typename + 'Created', validatedArgs, { id: addResults.id });
        return this.getRecord(req, { id: addResults.id }, query);
    }
    static async updateRecord(req, args = {}, query) {
        if (!req.user)
            throw error_1.default.loginRequiredError();
        //if it does not pass the access control, throw an error
        if (!await this.testPermissions('update', req, args, query)) {
            throw error_1.default.badPermissionsError();
        }
        //check if record exists
        const recordExistCount = await resolverHelper.countTableRows(this.__typename, [
            {
                fields: [
                    { field: "id", value: args.id }
                ]
            }
        ]);
        if (recordExistCount < 1) {
            throw error_1.default.generateError('Item not found', 404);
        }
        await resolverHelper.updateTableRow(this.__typename, Object.assign({}, args), {
            date_modified: null
        }, [{ fields: [{ field: "id", value: args.id }] }]);
        const returnData = this.getRecord(req, { id: args.id }, query);
        //check subscriptions table where companyUpdated and id = 1.
        const validatedArgs = {
            id: args.id
        };
        subscription_1.handleJqlSubscriptionTrigger(req, this, this.__typename + 'Updated', validatedArgs);
        return returnData;
    }
    static async deleteRecord(req, args = {}, query) {
        if (!req.user)
            throw error_1.default.loginRequiredError();
        //if it does not pass the access control, throw an error
        if (!await this.testPermissions('delete', req, args, query)) {
            throw error_1.default.badPermissionsError();
        }
        //first, fetch the requested query
        const requestedResults = await this.getRecord(req, { id: args.id }, query);
        //check subscriptions table where companyDeleted and id = 1.
        const validatedArgs = {
            id: args.id
        };
        await subscription_1.handleJqlSubscriptionTrigger(req, this, this.__typename + 'Deleted', validatedArgs);
        await resolverHelper.deleteTableRow(this.__typename, args, [
            {
                fields: [
                    { field: "id", value: args.id }
                ]
            }
        ]);
        //cleanup
        //also need to delete all subscriptions for this item
        await subscription_1.deleteJqlSubscription(req, this.__typename + 'Deleted', validatedArgs);
        //also need to delete all permissions attached to this item
        if (this.permissionsLink) {
            await resolverHelper.deleteTableRow(this.permissionsLink.__typename, args, [
                {
                    fields: [
                        { field: this.__typename, value: args.id }
                    ]
                }
            ]);
        }
        return requestedResults;
    }
}
exports.Service = Service;
Service.filterFieldsMap = {};
Service.filterFieldsKeyMap = { id: {} };
Service.hasKeys = true;
Service.sortFieldsMap = {};
Service.groupByFieldsMap = {};
Service.searchFieldsMap = {};
Service.isFilterRequired = false;
Service.generatePaginatorService = generators_1.generatePaginatorService;
Service.generateAccessorService = generators_1.generateAccessorService;
;
//# sourceMappingURL=service.js.map